/**************************************************************

   Program Title:  
   
   Program Name: WP12_KEY.c 		  Version: 0.01
   
   Author:  WR	  				  Date:  Febuary 29th, 2008
   
   Compiled with ImageCraft ICCAVR Standard V7.1
   
   Copyright 2004-2008, SOC Machines, Inc.  All rights reserved.
   
   
   
   This is a simple modual that is used to connect a PS/2 keyboard
   to your application.  It will poll the PS/2 port when you are
   looking to read a key.  If an interrupt is generated while the
   keyboard is being read, this will cause the program to behave
   incorrectly!  Also this program only provides limited support
   in the number of scan codes it understands.  For example
   it does not understand UPPER case letters.
   
****************************************************************/
/*
 * Copyright (C) 2008 by SOC Machines, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC MACHINES, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */



#include "WP12_KEY.h"
#include <dev/board.h>

/*
 *This array stores some of the keyboards scancodes
 */
unsigned char scancodes[][2]= {
0x0d,9,
0x0e,'|',
0x15,'q',
0x16,'1',
0x1a,'z',
0x1b,'s',
0x1c,'a',
0x1d,'w',
0x1e,'2',
0x21,'c',
0x22,'x',
0x23,'d',
0x24,'e',
0x25,'4',
0x26,'3',
0x29,' ',
0x2a,'v',
0x2b,'f',
0x2c,'t',
0x2d,'r',
0x2e,'5',
0x31,'n',
0x32,'b',
0x33,'h',
0x34,'g',
0x35,'y',
0x36,'6',
0x39,',',
0x3a,'m',
0x3b,'j',
0x3c,'u',
0x3d,'7',
0x3e,'8',
0x41,',',
0x42,'k',
0x43,'i',
0x44,'o',
0x45,'0',
0x46,'9',
0x49,'.',
0x4a,'-',
0x4b,'l',
0x4c,'',
0x4d,'p',
0x4e,'+',
0x52,'',
0x54,'',
0x55,'\\',
0x5a,13,
0x5b,'',
0x5d,'\'',
0x61,'<',
0x66,8,
0x69,'1',
0x6b,'4',
0x6c,'7',
0x70,'0',
0x71,',',
0x72,'2',
0x73,'5',
0x74,'6',
0x75,'8',
0x79,'+',
0x7a,'3',
0x7b,'-',
0x7c,'*',
0x7d,'9',
0,0
};

/*
 *Used as a flag to see if a scan code we recieved from the keybaord matches
 *those in our scancodes
 */
unsigned char valid;


/*
 *Sets up the pins the keyboard is connect to as inputs
 */
void InitKEY(void)
{
 	 //Set pins as inputs
	 DDRD &= ~(1<<CLK);
	 DDRD &= ~(1<<DATA);
}

/*
 *This function will simply poll the clk line until it is driven low by the
 *the keyboard.  It will then read in the data bits when clk is low.
 *
 *The function will then try to decode the scan code.
 *If the code read was not a valid code, then the function will
 *poll the keyboard again
 */
unsigned char getKey(void)
{
 	 unsigned char data;//stores the scancode
	 int bitCount;//counts the number of bits in the transaction (11)
 		 
		 
	 do{
		 data = 0x00;
		 //counts the 11 data bits
		 for(bitCount = 0;bitCount < 11; bitCount++)
		 {		 
		  		  while(PIND&(1<<CLK));//wait till we get a low
				  //ignore the start, parity & stop bits  
				  if((bitCount>0)&&(bitCount<9))
				  {
				   		if((PIND&(1<<DATA)))
				   					   data |= 1<<(bitCount-1);
				  
				  }
				  while(!(PIND&(1<<CLK)));//wait till we go high again
		 }
		 //try to decode the scancode
		 data = decode(data);
	  }while (!valid);//if we did not get one of our scancodes, repeat
		 	
	  return data;
}


/*
 *Will try to match the scancodes
 *Asserts valid when we are successful
 */
unsigned char decode(unsigned char code)
{
 		 int i;
		 valid = 0;
		 
		 //try to find the data
		 for (i = 0; (scancodes[i][0] != code) && scancodes[i][0]; i++);
		 
		 if (scancodes[i][0] == code)
		 {
		    code = scancodes[i][1];
		    valid = 1;
		 }
		 
		 return code;
}